# TODO: use sliding windows instead of chunking.
# the chunking method fails when part of a dupe string
# crosses the border between chunks

import colorama
from colorama import Fore
from tqdm import trange, tqdm
import os

colorama.init(autoreset=True)

DEFAULT_MIN = 5
DEFAULT_MAX = 10


def striplist(l):
    # clean out some unneeded chars
    return [x.strip(" \t\n\r") for x in l]


def compare(input):
    # main comparison function
    test = input[0]
    data = input[1]
    chunk_length = input[2]
    # print("data", data)
    found = data.find(test)
    if found != -1:
        words = test.split()
        # don't return matched chunks shorter than the current chunk
        # length, even if they are rounding remainder orphans from the
        # chunking process
        if len(words) >= chunk_length:
            return test


def make_chunks(lst, n):
    # Yield successive n-sized chunks from lst.
    for i in range(0, len(lst), n):
        yield lst[i : i + n]


def chunkit(str, chunk_length):
    # make chunks of strings the way we like
    chunks = []
    list = str.split()
    list = striplist(list)
    wordLists = make_chunks(list, chunk_length)
    for chunk in wordLists:
        if chunk != "":
            chunk = " ".join(chunk)
            chunks.append(chunk)
    return chunks


def run(chunk_length, text, dataset):
    dataset = dataset.replace(os.linesep, " ")
    testChunks = chunkit(text, chunk_length)
    # remove empty lines
    testChunks = list(filter(None, testChunks))
    results = []
    for testLine in testChunks:
        found = compare([testLine, dataset, chunk_length])
        if found != None:
            print("found", found)
            results.append(found)
    return results


def dupecheck(min=DEFAULT_MIN, max=DEFAULT_MAX, text=None, dataset=None, verbose=False):
    assert text != None
    assert dataset != None
    text = text.replace(os.linesep, "")
    matches = []
    for i in trange(max, min, -1):
        # print('text, i',text, i)
        res = run(i, text, dataset)
        if len(res) > 0:
            for r in res:
                if r not in matches:
                    matches.append(r)
    return matches



import pathlib
from setuptools import find_packages, setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name="dupecheck",
    version="0.6.3",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Jakob Anderson",
    author_email="fictionisfiction@gmail.com",
    license="MIT",
    url="https://github.com/spacerockzero/dupecheck",
    description="Check for duplicate word series between generated text and a dataset",
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    install_requires=["tqdm"],
    setup_requires=["pytest-runner"],
    tests_require=["pytest==4.4.1", "numpy"],
    test_suite="tests",
)



# Use sliding windows instead of chunking.
# the chunking method fails when part of a dupe string
# crosses the border between chunks
import os
import re
from tqdm import trange, tqdm

DEFAULT_MIN = 5
DEFAULT_MAX = 10


def clean_string(s):
    s = s.replace(os.linesep, " ")
    s = re.sub(r"[^\w\s]", "", s)
    s = s.strip()
    return s


def compare(text, data):
    text = clean_string(text)
    found = data.find(text)
    if found != -1:
        return text


def run(window_length, text, dataset):
    print("window: " + str(window_length))
    dataset = clean_string(dataset)
    results = []
    text_words = text.split()
    for num, t_word in tqdm(iterable=enumerate(text_words), total=len(text_words)):
        window_indexes = list(range(num, num + window_length))
        window_words = ""
        for i in window_indexes:
            if i < len(text_words):
                word = text_words[i]
                if word != None:
                    window_words += word + " "
        duplicate_text = compare(window_words, dataset)
        if duplicate_text != None:
            wrds = duplicate_text.split()
            if len(wrds) >= window_length:
                results.append(duplicate_text)
    return results


def dupecheck(min=DEFAULT_MIN, max=DEFAULT_MAX, text=None, dataset=None, verbose=False):
    assert text != None
    assert dataset != None
    min = min - 1
    dataset = clean_string(dataset)
    matches = set([])
    for i in range(max, min, -1):
        res = run(i, text, dataset)
        if len(res) > 0:
            matches.update(res)
    matchs = sorted(matches, key=len)
    return matches
